var EclipseDittoJavascriptClient = (function (exports, dittoJavascriptClientApi_1_0) {
    'use strict';

    /*!
     * Copyright (c) 2019 Contributors to the Eclipse Foundation
     *
     * See the NOTICE file(s) distributed with this work for additional
     * information regarding copyright ownership.
     *
     * This program and the accompanying materials are made available under the
     * terms of the Eclipse Public License 2.0 which is available at
     * http://www.eclipse.org/legal/epl-2.0
     *
     * SPDX-License-Identifier: EPL-2.0
     */
    class FetchRequester {
        doRequest(method, url, header, body) {
            return fetch(this.prepareRequest(method, url, header, body))
                .then(response => {
                const headers = new Map();
                response.headers.forEach((v, k) => headers.set(k, v));
                return response.json()
                    .then(json => ({ headers, status: response.status, body: json }), () => ({ headers, status: response.status, body: undefined }));
            });
        }
        /**
         * Builds a Request object to perform a fetch request with.
         *
         * @param method - The type of action to perform.
         * @param url - The Url to send the request to.
         * @param header - The headers of the request.
         * @param body - The payload to send with the request.
         * @return the builder.
         */
        prepareRequest(method, url, header, body) {
            const headers = new Headers();
            header.forEach((v, k) => headers.append(k, v));
            return new Request(url, {
                headers,
                method,
                body,
                mode: 'cors'
            });
        }
    }

    /*!
     * Copyright (c) 2019 Contributors to the Eclipse Foundation
     *
     * See the NOTICE file(s) distributed with this work for additional
     * information regarding copyright ownership.
     *
     * This program and the accompanying materials are made available under the
     * terms of the Eclipse Public License 2.0 which is available at
     * http://www.eclipse.org/legal/epl-2.0
     *
     * SPDX-License-Identifier: EPL-2.0
     */
    /**
     * Browser implementation of a web socket requester.
     */
    class FetchWebSocket {
        constructor(webSocket, webSocketUrl, handler) {
            this.webSocket = webSocket;
            this.webSocketUrl = webSocketUrl;
            this.handler = handler;
            this.connected = true;
            this.setHandles();
        }
        /**
         * Builds an instance of FetchWebSocket.
         *
         * @param url - The Url of the service.
         * @param handler - The handler that gets called for responses from the web socket.
         * @param authProviders - The auth providers to use.
         * @return a Promise for the web socket connection.
         */
        static buildInstance(url, handler, authProviders) {
            return new Promise(resolve => {
                const authenticatedUrl = dittoJavascriptClientApi_1_0.authenticateWithUrl(url, authProviders);
                const finalUrl = authenticatedUrl.toString();
                const webSocket = new WebSocket(finalUrl);
                webSocket.addEventListener('open', () => {
                    resolve(new FetchWebSocket(webSocket, finalUrl, handler));
                });
            });
        }
        executeCommand(request) {
            this.webSocket.send(request);
        }
        /**
         * Reestablishes a failed web socket connection.
         *
         * @param retry - The amount of time to wait to reconnect.
         * @return a Promise for the reestablished web socket connection.
         */
        reconnect(retry) {
            return new Promise((resolve, reject) => {
                this.webSocket = new WebSocket(this.webSocketUrl);
                this.webSocket.addEventListener('open', () => {
                    this.connected = true;
                    this.setHandles();
                    resolve(this);
                });
                setTimeout(() => {
                    if (this.connected) {
                        return;
                    }
                    if (retry <= 120000) {
                        console.log('Reconnect failed! Trying again!');
                        resolve(this.reconnect(retry * 2));
                    }
                    console.log('Reconnect failed!');
                    reject('Reconnect failed: Timed out');
                }, retry);
            });
        }
        /**
         * Sets up the handler so it receives events from the web socket.
         */
        setHandles() {
            this.webSocket.addEventListener('message', event => {
                this.handler.handleInput(event.data);
            });
            this.webSocket.addEventListener('close', () => {
                this.connected = false;
                this.handler.handleClose(Promise.resolve(this.reconnect(1000)));
            });
            this.webSocket.addEventListener('error', event => {
                this.handler.handleError(`WebSocket: ${event}`);
            });
        }
    }
    /**
     * Builder for the Browser implementation of a web socket.
     */
    class FetchWebSocketBuilder {
        constructor() {
        }
        withHandler(handler) {
            return FetchWebSocket.buildInstance(this.dittoUrl, handler, this.authProviders);
        }
        withConnectionDetails(url, authProviders) {
            this.authProviders = authProviders;
            this.dittoUrl = url;
            return this;
        }
    }

    /*!
     * Copyright (c) 2019 Contributors to the Eclipse Foundation
     *
     * See the NOTICE file(s) distributed with this work for additional
     * information regarding copyright ownership.
     *
     * This program and the accompanying materials are made available under the
     * terms of the Eclipse Public License 2.0 which is available at
     * http://www.eclipse.org/legal/epl-2.0
     *
     * SPDX-License-Identifier: EPL-2.0
     */
    /**
     * Starting point to build clients that can be used to get handles for browsers.
     */
    class DittoDomClient {
        /**
         * Returns a mutable builder with a fluent API for creating a Http-Ditto-Client.
         * The returned builder utilizes *Object scoping* to guide you through the building process.
         *
         * @return the builder.
         */
        static newHttpClient() {
            return dittoJavascriptClientApi_1_0.HttpClientBuilder.newBuilder(new FetchRequester());
        }
        /**
         * Returns a mutable builder with a fluent API for creating a Web-Socket-Ditto-Client.
         * The returned builder utilizes *Object scoping* to guide you through the building process.
         *
         * @return the builder.
         */
        static newWebSocketClient() {
            return dittoJavascriptClientApi_1_0.WebSocketClientBuilder.newBuilder(new FetchWebSocketBuilder());
        }
    }

    /*!
     * Copyright (c) 2019 Contributors to the Eclipse Foundation
     *
     * See the NOTICE file(s) distributed with this work for additional
     * information regarding copyright ownership.
     *
     * This program and the accompanying materials are made available under the
     * terms of the Eclipse Public License 2.0 which is available at
     * http://www.eclipse.org/legal/epl-2.0
     *
     * SPDX-License-Identifier: EPL-2.0
     */
    /**
     * Dom implementation of a Base64 encoder.
     */
    class DomBase64Encoder {
        encodeBase64(toEncode) {
            return btoa(toEncode);
        }
    }
    /**
     * Dom implementation of basic auth for HTTP connections.
     */
    class DomHttpBasicAuth extends dittoJavascriptClientApi_1_0.HttpBasicAuth {
        constructor(username, password, encoder) {
            super(username, password, encoder);
        }
        /**
         * Create basic authentication for HTTP connections.
         * @param username - The username.
         * @param password - the password.
         */
        static newInstance(username, password) {
            return new DomHttpBasicAuth(username, password, new DomBase64Encoder());
        }
    }
    /**
     * Dom implementation of basic auth for WebSocket connections.
     */
    class DomWebSocketBasicAuth extends dittoJavascriptClientApi_1_0.BasicAuth {
        constructor(username, password, encoder) {
            super(username, password, encoder);
        }
        /**
         * Create basic authentication for Http connections.
         * @param username - The username.
         * @param password - the password.
         */
        static newInstance(username, password) {
            return new DomWebSocketBasicAuth(username, password, new DomBase64Encoder());
        }
        authenticateWithHeaders(originalHeaders) {
            return originalHeaders;
        }
        authenticateWithUrl(originalUrl) {
            return originalUrl.withDomain(`${encodeURIComponent(this.username)}:${encodeURIComponent(this.password)}@${originalUrl.domain}`);
        }
    }

    exports.DittoDomClient = DittoDomClient;
    exports.DomBase64Encoder = DomBase64Encoder;
    exports.DomHttpBasicAuth = DomHttpBasicAuth;
    exports.DomWebSocketBasicAuth = DomWebSocketBasicAuth;
    exports.FetchRequester = FetchRequester;
    exports.FetchWebSocket = FetchWebSocket;
    exports.FetchWebSocketBuilder = FetchWebSocketBuilder;

    return exports;

}({}, EclipseDittoJavascriptClientApi));
//# sourceMappingURL=index.bundle.js.map
